from django.shortcuts import render, redirect, get_object_or_404
from django.http import HttpResponse, HttpResponseForbidden
from django.contrib.auth.decorators import login_required
from django.contrib import messages
from .models import Supplier
from django.core.paginator import Paginator
from django.db import IntegrityError

@login_required
def suppliers(request):
    if request.user.is_admin_user or request.user.is_super_user or request.user.is_inventory:
        try:
            suppliers_list = Supplier.objects.all()
            paginator = Paginator(suppliers_list, 10)  # Number of suppliers per page

            page_number = request.GET.get('page')
            page_obj = paginator.get_page(page_number)

            return render(request, "inventory/supplier/suppliers.html", {'page_obj': page_obj})
        except Exception as e:
            messages.error(request, f"An error occurred: {str(e)}")
            return redirect('inventory:suppliers')
    else:
        return HttpResponseForbidden("You are not authorized to view this page.")

@login_required
def add_supplier(request):
    if request.user.is_admin_user or request.user.is_super_user or request.user.is_inventory:
        if request.method == 'POST':
            try:
                supplier_name = request.POST.get('supplier_name')
                contact_name = request.POST.get('contact_name')
                email = request.POST.get('email')
                phone = request.POST.get('phone')
                address = request.POST.get('address')
                city = request.POST.get('city')
                state = request.POST.get('state')
                postal_code = request.POST.get('postal_code')
                country = request.POST.get('country')
                kra_pin = request.POST.get('kra_pin')

                supplier = Supplier(
                    supplier_name=supplier_name,
                    contact_name=contact_name,
                    email=email,
                    phone=phone,
                    address=address,
                    city=city,
                    state=state,
                    postal_code=postal_code,
                    country=country,
                    kra_pin=kra_pin
                )
                supplier.save()
                messages.success(request, 'Supplier added successfully.')
                return redirect('inventory:suppliers')
            except IntegrityError as e:
                messages.error(request, f"Data integrity error: {str(e)}")
            except Exception as e:
                messages.error(request, f"An error occurred: {str(e)}")
        return render(request, "inventory/supplier/add_supplier.html")
    else:
        return HttpResponseForbidden("You are not authorized to view this page.")

@login_required
def edit_supplier(request, supplier_id):
    if request.user.is_admin_user or request.user.is_super_user or request.user.is_inventory:
        try:
            supplier = get_object_or_404(Supplier, id=supplier_id)
            if request.method == 'POST':
                supplier.supplier_name = request.POST.get('supplier_name')
                supplier.contact_name = request.POST.get('contact_name')
                supplier.email = request.POST.get('email')
                supplier.phone = request.POST.get('phone')
                supplier.address = request.POST.get('address')
                supplier.city = request.POST.get('city')
                supplier.state = request.POST.get('state')
                supplier.postal_code = request.POST.get('postal_code')
                supplier.country = request.POST.get('country')
                supplier.kra_pin = request.POST.get('kra_pin')

                supplier.save()
                messages.success(request, 'Supplier updated successfully.')
                return redirect('inventory:suppliers')
            return render(request, "inventory/supplier/edit_supplier.html", {'supplier': supplier})
        except IntegrityError as e:
            messages.error(request, f"Data integrity error: {str(e)}")
            return redirect('inventory:suppliers')
        except Exception as e:
            messages.error(request, f"An error occurred: {str(e)}")
            return redirect('inventory:suppliers')
    else:
        return HttpResponseForbidden("You are not authorized to view this page.")

@login_required
def delete_supplier(request, supplier_id):
    if request.user.is_admin_user or request.user.is_super_user or request.user.is_inventory:
        try:
            supplier = get_object_or_404(Supplier, id=supplier_id)
            supplier_name = supplier.supplier_name
            supplier.delete()
            messages.success(request, f"Supplier '{supplier_name}' has been deleted successfully.")
        except Exception as e:
            messages.error(request, f"An error occurred while deleting the supplier: {str(e)}")
        return redirect("inventory:suppliers")
    else:
        return HttpResponseForbidden("You are not authorized to view this page.")